import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class offers a set of static methods that facilitate the construction of
 * lists of buttons, radio buttons and check boxes. Methods are overloaded for
 * flexibility; one can create buttons with all possible attributes or more
 * simple ones. Methods are offered too that build JPanels with properly layed
 * out sets of buttons.
 * @author Coti
 */
public class AuxButton {
    
    /**
     * This method can be used internall and externally in order to show an alert.
     * @param owner The component on which the error dialog will  be centered
     * @param message Actual text of message
     * @param title Title that is seen on the upper bar
     */
    public static void errMess(Component owner, String message, String title) {
        JOptionPane.showMessageDialog(	owner,
                message,
                title,
                JOptionPane.ERROR_MESSAGE);
    }
    
    /**
     * This is the most complete method for button creation. It admits arguments
     * of type ArrayList (one might use Collections.addAll(arraylist,array) in order
     * to create them conveniently).
     * @param names A list of Strings used as names for buttons
     * @param listeners A list of ActionListener, one for each button
     * @param keyEvents The mnemonics for this set, a list of Strings
     * @param actionCmds The action commands, also a list of Strings
     * @param icons A list of image icons, one for each button
     * @return An ArrayList<JButton> with all buttons properly built
     */
    public static ArrayList<JButton> createButtonList(ArrayList<String>names,
            ArrayList<ActionListener>listeners,
            ArrayList<Integer> keyEvents,
            ArrayList<String> actionCmds,
            ArrayList<ImageIcon> icons) {
        ArrayList<JButton> bList = new ArrayList<JButton>();
        if (null == names) {
            errMess(null,"No names for buttons","AuxButton - fatal error");
            System.exit(1);
        }
        int number = names.size();
        JButton temp;
        bList = new ArrayList<JButton>();
        for(int i=0;i<number;i++) {
            temp = new JButton(names.get(i));
            if (null != listeners)
                temp.addActionListener(listeners.get(i));
            if (null != keyEvents)
                temp.setMnemonic(keyEvents.get(i));
            if (null != actionCmds)
                temp.setActionCommand(actionCmds.get(i));
            if (null != icons)
                temp.setIcon(icons.get(i));
            bList.add(temp);
        }
        return bList;
    }
    
    /**
     * A more simple version of the previous version. A different ActionListener
     * can be supplied for each button.
     * @param names A list of Strings used as names for buttons
     * @param listeners A list of ActionListener, one for each button
     * @return An ArrayList<JButton> with all buttons properly built
     */
    public static ArrayList<JButton> createButtonList(ArrayList<String>names,
            ArrayList<ActionListener>listeners) {
        ArrayList<JButton> bList = new ArrayList<JButton>();
        if (null == names) {
            errMess(null,"No names for buttons","AuxButton - fatal error");
            System.exit(1);
        }
        int number = names.size();
        JButton temp;
        bList = new ArrayList<JButton>();
        for(int i=0;i<number;i++) {
            temp = new JButton(names.get(i));
            if (null != listeners)
                temp.addActionListener(listeners.get(i));
            bList.add(temp);
        }
        return bList;
    }
    
    /**
     * Only one ActionListener is used for all buttons.
     * @param names A list of Strings used as names for buttons
     * @param listener A single ActionListener, to be used by all buttons
     * @return An ArrayList<JButton> with all buttons properly built
     */
    public static ArrayList<JButton> createButtonList(ArrayList<String>names,
            ActionListener listener) {
        ArrayList<JButton> bList = new ArrayList<JButton>();
        if (null == names) {
            errMess(null,"No names for buttons","AuxButton - fatal error");
            System.exit(1);
        }
        if (null == listener) {
            errMess(null,"No listener for buttons","AuxButton - fatal error");
            System.exit(1);
        }
        int number = names.size();
        JButton temp;
        bList = new ArrayList<JButton>();
        for(int i=0;i<number;i++) {
            temp = new JButton(names.get(i));
            temp.addActionListener(listener);
            bList.add(temp);
        }
        return bList;
    }
    
    /**
     * This method produces a set of radio buttons. Each one can have a different
     * ActionListener.
     * @param names A list of Strings used as names for radio buttons
     * @param listeners A list of ActionListener, one for each button
     * @param selected An int that states which of the buttons must be selected
     * @return A list of radio buttons created as specified by the arguments
     */
    public static  ArrayList<JRadioButton> createRBList(ArrayList<String> names,
            ArrayList<ActionListener> listeners,
            int selected) {
        if (null == names) {
            errMess( null,"No names for radio buttons",
                    "AuxButton - fatal error");
            System.exit(1);
        }
        int number = names.size();
        ButtonGroup bg = new ButtonGroup();
        JRadioButton temp;
        ArrayList<JRadioButton> rbList = new ArrayList<JRadioButton>();
        for(int i=0;i<number;i++) {
            temp = new JRadioButton(names.get(i));
            if (null != listeners)
                temp.addActionListener(listeners.get(i));
            temp.setSelected(selected == i ? true:false);
            rbList.add(temp);
            bg.add(temp);
        }
        return rbList;
    }
    
    /**
     * Only one ActionListener is used for all buttons.
     * @param names A list of Strings used as names for radio buttons
     * @param listener A single ActionListener, to be used by all buttons
     * @param selected An int that states which of the buttons must be selected
     * @return An array list that contains a set of properly created radio buttons.
     */
    public static  ArrayList<JRadioButton> createRBList(ArrayList<String> names,
            ActionListener listener,
            int selected) {
        if (null == names) {
            errMess( null,"No names for radio buttons",
                    "AuxButton - fatal error");
            System.exit(1);
        }
        if (null == listener) {
            errMess(null,"No listener for radio buttons","AuxButton - fatal error");
            System.exit(1);
        }
        int number = names.size();
        ButtonGroup bg = new ButtonGroup();
        JRadioButton temp;
        ArrayList<JRadioButton> rbList = new ArrayList<JRadioButton>();
        for(int i=0;i<number;i++) {
            temp = new JRadioButton(names.get(i));
            if (null != listener)
                temp.addActionListener(listener);
            temp.setSelected(selected == i ? true : false);
            rbList.add(temp);
            bg.add(temp);
        }
        return rbList;
    }
    
    /**
     * This method produces a list of check boxes, built according to arguments.
     * A different listener may  be used for each check box.
     * @param names A list with the names to be used for the check boxes.
     * @param listeners A list of ArrayListener, one for each check box.
     * @param selecteds The specification of the status of each of the check boxes.
     * @return An array list that contains a set of properly created check boxes.
     */
    public static  ArrayList<JCheckBox> createCBList(ArrayList<String> names,
            ArrayList<ActionListener> listeners,
            ArrayList<Boolean> selecteds) {
        if (null == names) {
            errMess(null, "No names for check boxes",
                    "AuxButton - fatal error");
            System.exit(1);
        }
        int number = names.size();
        JCheckBox temp;
        ArrayList<JCheckBox> cbList = new ArrayList<JCheckBox>();
        for(int i=0;i<number;i++) {
            temp = new JCheckBox(names.get(i));
            if (null != listeners)
                temp.addActionListener(listeners.get(i));
            if (null != selecteds)
                temp.setSelected(selecteds.get(i));
            cbList.add(temp);
        }
        return cbList;
    }
    
    /**
     * This method produces a list of check boxes, built according to arguments. Only
     * one listener is used for all check boxes.
     * @param names The list of names to be used for this set of check boxes.
     * @param listener A single listener, to be used for all check boxes.
     * @param selecteds The selection status of all check boxes.
     * @return A list of check boxes built according to the arguments.
     */
    public static  ArrayList<JCheckBox> createCBList(ArrayList<String> names,
            ActionListener listener,
            ArrayList<Boolean> selecteds) {
        if (null == names) {
            errMess(null, "No names for check boxes",
                    "AuxButton - fatal error");
            System.exit(1);
        }
        if (null == listener) {
            errMess(null,"No listener for check boxes","AuxButton - fatal error");
            System.exit(1);
        }
        int number = names.size();
        JCheckBox temp;
        ArrayList<JCheckBox> cbList = new ArrayList<JCheckBox>();
        for(int i=0;i<number;i++) {
            temp = new JCheckBox(names.get(i));
            if (null != listener)
                temp.addActionListener(listener);
            if (null != selecteds)
                temp.setSelected(selecteds.get(i));
            cbList.add(temp);
        }
        return cbList;
    }
    
    
    /**
     *
     * @param d Descriptor of the SpringLayout that will be used
     * @param comp The component that will be placed
     * @param xPos Upper left corner of said component
     * @param yPos
     * @param width Dimensions of the component
     * @param height
     */
    private static void pws(	SpringLayout d,
            Component comp,
            int xPos,
            int yPos,
            int width,
            int height) {
        Dimension dim = new Dimension(width, height);
        comp.setPreferredSize(dim);
        comp.setMaximumSize(dim);
        
        SpringLayout.Constraints cons = d.getConstraints(comp);
        cons.setX(Spring.constant(xPos));
        cons.setY(Spring.constant(yPos));
    }
    
    /**
     * This methods produces a panel that contains the buttons received as
     * an argument, properly spaced.
     * @param bList The list of buttons that will be layed out in a panel
     * @return A panel with the buttons properly placed
     */
    public static JPanel panelWithButtons(ArrayList<JButton> bList) {
        JPanel bPanel = new JPanel();
        SpringLayout sl = new SpringLayout();
        bPanel.setLayout(sl);
        JButton b        = null;
        int yPos         = 6;
        int buttonHeight = 0;
        int buttonWidth = 0;
        FontMetrics fm = null;
        int minWidth = 200;
        int minHeight = 0;
        b = bList.get(0);
        fm = b.getFontMetrics(b.getFont());
        
        for(int i=0;i<bList.size();i++) {
            b = bList.get(i);
            buttonHeight = b.getHeight() + 10;
            buttonWidth = fm.stringWidth(b.getText())+50;
            pws(sl,b, 6, yPos, buttonWidth, buttonHeight);
            yPos += buttonHeight+10;
            bPanel.add(b);
            if (buttonWidth > minWidth)
                minWidth=buttonWidth;
        }
        minHeight = yPos;
        Dimension dim = new Dimension(minWidth,minHeight);
        bPanel.setPreferredSize(dim);
        bPanel.setMaximumSize(dim);
        return bPanel;
    }
    
    /**
     * This methods produces a panel that contains the buttons received as
     * an argument, properly spaced according to the size of their icons.
     * @param bList The list of buttons that will be layed out in a panel
     * @return A panel with the buttons properly placed, taking icons into account.
     */
    public static JPanel panelWithIconButtons(ArrayList<JButton> bList) {
        JPanel bPanel = new JPanel();
        SpringLayout sl = new SpringLayout();
        bPanel.setLayout(sl);
        JButton b        = null;
        int yPos         = 6;
        int buttonHeight = 0;
        int buttonWidth = 0;
        FontMetrics fm = null;
        int minWidth = 200;
        int minHeight = 0;
        b = bList.get(0);
        fm = b.getFontMetrics(b.getFont());
        
        for(int i=0;i<bList.size();i++) {
            b = bList.get(i);
            buttonHeight = b.getIcon().getIconHeight() + 10;
            buttonWidth = fm.stringWidth(b.getText())+50;
            pws(sl,b, 6, yPos, buttonWidth, buttonHeight);
            yPos += buttonHeight+10;
            bPanel.add(b);
            if (buttonWidth > minWidth)
                minWidth=buttonWidth;
        }
        minHeight = yPos;
        Dimension dim = new Dimension(minWidth,minHeight);
        bPanel.setPreferredSize(dim);
        bPanel.setMaximumSize(dim);
        return bPanel;
    }
    
    /**
     * This methods produces a panel that contains the radio buttons received as
     * an argument, properly spaced.
     * @param bList The list of buttons that will be layed out in a panel
     * @return A panel with the radio buttons properly placed
     */
    public static JPanel panelWithRadioButtons(ArrayList<JRadioButton> bList) {
        JPanel bPanel = new JPanel();
        SpringLayout sl = new SpringLayout();
        bPanel.setLayout(sl);
        JRadioButton b        =  null;
        int yPos         =     6;
        int buttonHeight =    18;
        int buttonWidth  =     0;
        FontMetrics fm = null;
        int minWidth = 200;
        int minHeight = 0;
        
        b = bList.get(0);
        fm = b.getFontMetrics(b.getFont());
        
        for(int i=0;i<bList.size();i++) {
            b = bList.get(i);
            
            buttonWidth = fm.stringWidth(b.getText())+50;
            pws(sl,b, 6, yPos, buttonWidth, buttonHeight);
            yPos += buttonHeight+10;
            bPanel.add(b);
            if (buttonWidth > minWidth)
                minWidth=buttonWidth;
        }
        minHeight = yPos;
        Dimension dim = new Dimension(minWidth,minHeight);
        bPanel.setPreferredSize(dim);
        bPanel.setMaximumSize(dim);
        return bPanel;
    }
    
    /**
     * This method produces a panel that contains the check boxes received as
     * an argument, properly spaced.
     * @param bList The list of buttons that will be layed out in a panel
     * @return A panel with the check boxes properly placed
     */
    public static JPanel panelWithCheckBoxes(ArrayList<JCheckBox> bList) {
        JPanel bPanel = new JPanel();
        SpringLayout sl = new SpringLayout();
        bPanel.setLayout(sl);
        JCheckBox b        =  null;
        int yPos         =     6;
        int buttonHeight =    18;
        int buttonWidth  =     0;
        FontMetrics fm = null;
        int minWidth = 200;
        int minHeight = 0;
        b = bList.get(0);
        fm = b.getFontMetrics(b.getFont());
        
        for(int i=0;i<bList.size();i++) {
            b = bList.get(i);
            buttonWidth = fm.stringWidth(b.getText())+50;
            pws(sl,b, 6, yPos, buttonWidth, buttonHeight);
            yPos += buttonHeight+10;
            bPanel.add(b);
            if (buttonWidth > minWidth)
                minWidth=buttonWidth;
        }
        minHeight = yPos;
        Dimension dim = new Dimension(minWidth,minHeight);
        bPanel.setPreferredSize(dim);
        bPanel.setMaximumSize(dim);
        return bPanel;
    }
    
    /**
     * This methods produces a list that contains the selection status of the check
     * boxes it receives as an argument.
     * @param cb A list of check boxes
     * @return A list of selection states of these check boxes
     */
    public static boolean[] getSelected(ArrayList<JCheckBox> cb) {
        int dim = cb.size();
        if (0 == dim) {
            errMess(null, "No checkboxes in list","Empty list");
            return null;
        }
        boolean[] r = new boolean[cb.size()];
        for(int i=0;i<dim;i++)
            r[i] = cb.get(i).isSelected();
        return r;
    }
    
    /**
     * This method indicates which of the radio buttons it receives as an argument
     * has been selected.
     * @param rb The list of buttons that will be layed out in a panel
     * @return The index of the selected radio button or -1
     */
    public static int getSelected(ArrayList<JRadioButton> rb) {
        int selectedButton = -1;
        int dim = rb.size();
        if (0 == dim) {
            errMess(null, "No radio buttons in list","Empty list");
            return selectedButton;
        }
        for(int i=0;i<dim;i++)
            if(rb.get(i).isSelected()) {
            selectedButton = i;
            break;
            }
        return selectedButton;
    }
}